/*
 * Copyright (c) 2013 PADL Software Pty Ltd.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. Redistributions in any form must be accompanied by information on
 *    how to obtain complete source code for the gss_browserid software
 *    and any accompanying software that uses the gss_browserid software.
 *    The source code must either be included in the distribution or be
 *    available for no more than the cost of distribution plus a nominal
 *    fee, and must be freely redistributable under reasonable conditions.
 *    For an executable file, complete source code means the source code
 *    for all modules it contains. It does not include source code for
 *    modules or files that typically accompany the major components of
 *    the operating system on which the executable file runs.
 *
 * THIS SOFTWARE IS PROVIDED BY PADL SOFTWARE ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 * NON-INFRINGEMENT, ARE DISCLAIMED. IN NO EVENT SHALL PADL SOFTWARE
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "bid_private.h"

/*
 * Constants from RFC 5114.
 */

/*
 * 2.1 1024-bit MODP Group with 160-bit Prime Order Subgroup
 */
static const unsigned char
_BIDDH_RFC5114_1024_P[] = {
    0xb1, 0x0b, 0x8f, 0x96, 0xa0, 0x80, 0xe0, 0x1d,
    0xde, 0x92, 0xde, 0x5e, 0xae, 0x5d, 0x54, 0xec,
    0x52, 0xc9, 0x9f, 0xbc, 0xfb, 0x06, 0xa3, 0xc6,
    0x9a, 0x6a, 0x9d, 0xca, 0x52, 0xd2, 0x3b, 0x61,
    0x60, 0x73, 0xe2, 0x86, 0x75, 0xa2, 0x3d, 0x18,
    0x98, 0x38, 0xef, 0x1e, 0x2e, 0xe6, 0x52, 0xc0,
    0x13, 0xec, 0xb4, 0xae, 0xa9, 0x06, 0x11, 0x23,
    0x24, 0x97, 0x5c, 0x3c, 0xd4, 0x9b, 0x83, 0xbf,
    0xac, 0xcb, 0xdd, 0x7d, 0x90, 0xc4, 0xbd, 0x70,
    0x98, 0x48, 0x8e, 0x9c, 0x21, 0x9a, 0x73, 0x72,
    0x4e, 0xff, 0xd6, 0xfa, 0xe5, 0x64, 0x47, 0x38,
    0xfa, 0xa3, 0x1a, 0x4f, 0xf5, 0x5b, 0xcc, 0xc0,
    0xa1, 0x51, 0xaf, 0x5f, 0x0d, 0xc8, 0xb4, 0xbd,
    0x45, 0xbf, 0x37, 0xdf, 0x36, 0x5c, 0x1a, 0x65,
    0xe6, 0x8c, 0xfd, 0xa7, 0x6d, 0x4d, 0xa7, 0x08,
    0xdf, 0x1f, 0xb2, 0xbc, 0x2e, 0x4a, 0x43, 0x71,
};

static const unsigned char
_BIDDH_RFC5114_1024_G[] = {
    0xa4, 0xd1, 0xcb, 0xd5, 0xc3, 0xfd, 0x34, 0x12,
    0x67, 0x65, 0xa4, 0x42, 0xef, 0xb9, 0x99, 0x05,
    0xf8, 0x10, 0x4d, 0xd2, 0x58, 0xac, 0x50, 0x7f,
    0xd6, 0x40, 0x6c, 0xff, 0x14, 0x26, 0x6d, 0x31,
    0x26, 0x6f, 0xea, 0x1e, 0x5c, 0x41, 0x56, 0x4b,
    0x77, 0x7e, 0x69, 0x0f, 0x55, 0x04, 0xf2, 0x13,
    0x16, 0x02, 0x17, 0xb4, 0xb0, 0x1b, 0x88, 0x6a,
    0x5e, 0x91, 0x54, 0x7f, 0x9e, 0x27, 0x49, 0xf4,
    0xd7, 0xfb, 0xd7, 0xd3, 0xb9, 0xa9, 0x2e, 0xe1,
    0x90, 0x9d, 0x0d, 0x22, 0x63, 0xf8, 0x0a, 0x76,
    0xa6, 0xa2, 0x4c, 0x08, 0x7a, 0x09, 0x1f, 0x53,
    0x1d, 0xbf, 0x0a, 0x01, 0x69, 0xb6, 0xa2, 0x8a,
    0xd6, 0x62, 0xa4, 0xd1, 0x8e, 0x73, 0xaf, 0xa3,
    0x2d, 0x77, 0x9d, 0x59, 0x18, 0xd0, 0x8b, 0xc8,
    0x85, 0x8f, 0x4d, 0xce, 0xf9, 0x7c, 0x2a, 0x24,
    0x85, 0x5e, 0x6e, 0xeb, 0x22, 0xb3, 0xb2, 0xe5,
};

/*
 * 2.3 2048-bit MODP Group with 256-bit Prime Order Subgroup
 */
static const unsigned char
_BIDDH_RFC5114_2048_P[] = {
    0x87, 0xa8, 0xe6, 0x1d, 0xb4, 0xb6, 0x66, 0x3c,
    0xff, 0xbb, 0xd1, 0x9c, 0x65, 0x19, 0x59, 0x99,
    0x8c, 0xee, 0xf6, 0x08, 0x66, 0x0d, 0xd0, 0xf2,
    0x5d, 0x2c, 0xee, 0xd4, 0x43, 0x5e, 0x3b, 0x00,
    0xe0, 0x0d, 0xf8, 0xf1, 0xd6, 0x19, 0x57, 0xd4,
    0xfa, 0xf7, 0xdf, 0x45, 0x61, 0xb2, 0xaa, 0x30,
    0x16, 0xc3, 0xd9, 0x11, 0x34, 0x09, 0x6f, 0xaa,
    0x3b, 0xf4, 0x29, 0x6d, 0x83, 0x0e, 0x9a, 0x7c,
    0x20, 0x9e, 0x0c, 0x64, 0x97, 0x51, 0x7a, 0xbd,
    0x5a, 0x8a, 0x9d, 0x30, 0x6b, 0xcf, 0x67, 0xed,
    0x91, 0xf9, 0xe6, 0x72, 0x5b, 0x47, 0x58, 0xc0,
    0x22, 0xe0, 0xb1, 0xef, 0x42, 0x75, 0xbf, 0x7b,
    0x6c, 0x5b, 0xfc, 0x11, 0xd4, 0x5f, 0x90, 0x88,
    0xb9, 0x41, 0xf5, 0x4e, 0xb1, 0xe5, 0x9b, 0xb8,
    0xbc, 0x39, 0xa0, 0xbf, 0x12, 0x30, 0x7f, 0x5c,
    0x4f, 0xdb, 0x70, 0xc5, 0x81, 0xb2, 0x3f, 0x76,
    0xb6, 0x3a, 0xca, 0xe1, 0xca, 0xa6, 0xb7, 0x90,
    0x2d, 0x52, 0x52, 0x67, 0x35, 0x48, 0x8a, 0x0e,
    0xf1, 0x3c, 0x6d, 0x9a, 0x51, 0xbf, 0xa4, 0xab,
    0x3a, 0xd8, 0x34, 0x77, 0x96, 0x52, 0x4d, 0x8e,
    0xf6, 0xa1, 0x67, 0xb5, 0xa4, 0x18, 0x25, 0xd9,
    0x67, 0xe1, 0x44, 0xe5, 0x14, 0x05, 0x64, 0x25,
    0x1c, 0xca, 0xcb, 0x83, 0xe6, 0xb4, 0x86, 0xf6,
    0xb3, 0xca, 0x3f, 0x79, 0x71, 0x50, 0x60, 0x26,
    0xc0, 0xb8, 0x57, 0xf6, 0x89, 0x96, 0x28, 0x56,
    0xde, 0xd4, 0x01, 0x0a, 0xbd, 0x0b, 0xe6, 0x21,
    0xc3, 0xa3, 0x96, 0x0a, 0x54, 0xe7, 0x10, 0xc3,
    0x75, 0xf2, 0x63, 0x75, 0xd7, 0x01, 0x41, 0x03,
    0xa4, 0xb5, 0x43, 0x30, 0xc1, 0x98, 0xaf, 0x12,
    0x61, 0x16, 0xd2, 0x27, 0x6e, 0x11, 0x71, 0x5f,
    0x69, 0x38, 0x77, 0xfa, 0xd7, 0xef, 0x09, 0xca,
    0xdb, 0x09, 0x4a, 0xe9, 0x1e, 0x1a, 0x15, 0x97,
};

static const unsigned char
_BIDDH_RFC5114_2048_G[] = {
    0x3f, 0xb3, 0x2c, 0x9b, 0x73, 0x13, 0x4d, 0x0b,
    0x2e, 0x77, 0x50, 0x66, 0x60, 0xed, 0xbd, 0x48,
    0x4c, 0xa7, 0xb1, 0x8f, 0x21, 0xef, 0x20, 0x54,
    0x07, 0xf4, 0x79, 0x3a, 0x1a, 0x0b, 0xa1, 0x25,
    0x10, 0xdb, 0xc1, 0x50, 0x77, 0xbe, 0x46, 0x3f,
    0xff, 0x4f, 0xed, 0x4a, 0xac, 0x0b, 0xb5, 0x55,
    0xbe, 0x3a, 0x6c, 0x1b, 0x0c, 0x6b, 0x47, 0xb1,
    0xbc, 0x37, 0x73, 0xbf, 0x7e, 0x8c, 0x6f, 0x62,
    0x90, 0x12, 0x28, 0xf8, 0xc2, 0x8c, 0xbb, 0x18,
    0xa5, 0x5a, 0xe3, 0x13, 0x41, 0x00, 0x0a, 0x65,
    0x01, 0x96, 0xf9, 0x31, 0xc7, 0x7a, 0x57, 0xf2,
    0xdd, 0xf4, 0x63, 0xe5, 0xe9, 0xec, 0x14, 0x4b,
    0x77, 0x7d, 0xe6, 0x2a, 0xaa, 0xb8, 0xa8, 0x62,
    0x8a, 0xc3, 0x76, 0xd2, 0x82, 0xd6, 0xed, 0x38,
    0x64, 0xe6, 0x79, 0x82, 0x42, 0x8e, 0xbc, 0x83,
    0x1d, 0x14, 0x34, 0x8f, 0x6f, 0x2f, 0x91, 0x93,
    0xb5, 0x04, 0x5a, 0xf2, 0x76, 0x71, 0x64, 0xe1,
    0xdf, 0xc9, 0x67, 0xc1, 0xfb, 0x3f, 0x2e, 0x55,
    0xa4, 0xbd, 0x1b, 0xff, 0xe8, 0x3b, 0x9c, 0x80,
    0xd0, 0x52, 0xb9, 0x85, 0xd1, 0x82, 0xea, 0x0a,
    0xdb, 0x2a, 0x3b, 0x73, 0x13, 0xd3, 0xfe, 0x14,
    0xc8, 0x48, 0x4b, 0x1e, 0x05, 0x25, 0x88, 0xb9,
    0xb7, 0xd2, 0xbb, 0xd2, 0xdf, 0x01, 0x61, 0x99,
    0xec, 0xd0, 0x6e, 0x15, 0x57, 0xcd, 0x09, 0x15,
    0xb3, 0x35, 0x3b, 0xbb, 0x64, 0xe0, 0xec, 0x37,
    0x7f, 0xd0, 0x28, 0x37, 0x0d, 0xf9, 0x2b, 0x52,
    0xc7, 0x89, 0x14, 0x28, 0xcd, 0xc6, 0x7e, 0xb6,
    0x18, 0x4b, 0x52, 0x3d, 0x1d, 0xb2, 0x46, 0xc3,
    0x2f, 0x63, 0x07, 0x84, 0x90, 0xf0, 0x0e, 0xf8,
    0xd6, 0x47, 0xd1, 0x48, 0xd4, 0x79, 0x54, 0x51,
    0x5e, 0x23, 0x27, 0xcf, 0xef, 0x98, 0xc5, 0x82,
    0x66, 0x4b, 0x4c, 0x0f, 0x6c, 0xc4, 0x16, 0x59,
};

/*
 * 4. 3072-bit MODP Group (Group 15)
 */
static const unsigned char
_BIDDH_RFC3526_3072_P[] = {
    0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
    0xc9, 0x0f, 0xda, 0xa2, 0x21, 0x68, 0xc2, 0x34,
    0xc4, 0xc6, 0x62, 0x8b, 0x80, 0xdc, 0x1c, 0xd1,
    0x29, 0x02, 0x4e, 0x08, 0x8a, 0x67, 0xcc, 0x74,
    0x02, 0x0b, 0xbe, 0xa6, 0x3b, 0x13, 0x9b, 0x22,
    0x51, 0x4a, 0x08, 0x79, 0x8e, 0x34, 0x04, 0xdd,
    0xef, 0x95, 0x19, 0xb3, 0xcd, 0x3a, 0x43, 0x1b,
    0x30, 0x2b, 0x0a, 0x6d, 0xf2, 0x5f, 0x14, 0x37,
    0x4f, 0xe1, 0x35, 0x6d, 0x6d, 0x51, 0xc2, 0x45,
    0xe4, 0x85, 0xb5, 0x76, 0x62, 0x5e, 0x7e, 0xc6,
    0xf4, 0x4c, 0x42, 0xe9, 0xa6, 0x37, 0xed, 0x6b,
    0x0b, 0xff, 0x5c, 0xb6, 0xf4, 0x06, 0xb7, 0xed,
    0xee, 0x38, 0x6b, 0xfb, 0x5a, 0x89, 0x9f, 0xa5,
    0xae, 0x9f, 0x24, 0x11, 0x7c, 0x4b, 0x1f, 0xe6,
    0x49, 0x28, 0x66, 0x51, 0xec, 0xe4, 0x5b, 0x3d,
    0xc2, 0x00, 0x7c, 0xb8, 0xa1, 0x63, 0xbf, 0x05,
    0x98, 0xda, 0x48, 0x36, 0x1c, 0x55, 0xd3, 0x9a,
    0x69, 0x16, 0x3f, 0xa8, 0xfd, 0x24, 0xcf, 0x5f,
    0x83, 0x65, 0x5d, 0x23, 0xdc, 0xa3, 0xad, 0x96,
    0x1c, 0x62, 0xf3, 0x56, 0x20, 0x85, 0x52, 0xbb,
    0x9e, 0xd5, 0x29, 0x07, 0x70, 0x96, 0x96, 0x6d,
    0x67, 0x0c, 0x35, 0x4e, 0x4a, 0xbc, 0x98, 0x04,
    0xf1, 0x74, 0x6c, 0x08, 0xca, 0x18, 0x21, 0x7c,
    0x32, 0x90, 0x5e, 0x46, 0x2e, 0x36, 0xce, 0x3b,
    0xe3, 0x9e, 0x77, 0x2c, 0x18, 0x0e, 0x86, 0x03,
    0x9b, 0x27, 0x83, 0xa2, 0xec, 0x07, 0xa2, 0x8f,
    0xb5, 0xc5, 0x5d, 0xf0, 0x6f, 0x4c, 0x52, 0xc9,
    0xde, 0x2b, 0xcb, 0xf6, 0x95, 0x58, 0x17, 0x18,
    0x39, 0x95, 0x49, 0x7c, 0xea, 0x95, 0x6a, 0xe5,
    0x15, 0xd2, 0x26, 0x18, 0x98, 0xfa, 0x05, 0x10,
    0x15, 0x72, 0x8e, 0x5a, 0x8a, 0xaa, 0xc4, 0x2d,
    0xad, 0x33, 0x17, 0x0d, 0x04, 0x50, 0x7a, 0x33,
    0xa8, 0x55, 0x21, 0xab, 0xdf, 0x1c, 0xba, 0x64,
    0xec, 0xfb, 0x85, 0x04, 0x58, 0xdb, 0xef, 0x0a,
    0x8a, 0xea, 0x71, 0x57, 0x5d, 0x06, 0x0c, 0x7d,
    0xb3, 0x97, 0x0f, 0x85, 0xa6, 0xe1, 0xe4, 0xc7,
    0xab, 0xf5, 0xae, 0x8c, 0xdb, 0x09, 0x33, 0xd7,
    0x1e, 0x8c, 0x94, 0xe0, 0x4a, 0x25, 0x61, 0x9d,
    0xce, 0xe3, 0xd2, 0x26, 0x1a, 0xd2, 0xee, 0x6b,
    0xf1, 0x2f, 0xfa, 0x06, 0xd9, 0x8a, 0x08, 0x64,
    0xd8, 0x76, 0x02, 0x73, 0x3e, 0xc8, 0x6a, 0x64,
    0x52, 0x1f, 0x2b, 0x18, 0x17, 0x7b, 0x20, 0x0c,
    0xbb, 0xe1, 0x17, 0x57, 0x7a, 0x61, 0x5d, 0x6c,
    0x77, 0x09, 0x88, 0xc0, 0xba, 0xd9, 0x46, 0xe2,
    0x08, 0xe2, 0x4f, 0xa0, 0x74, 0xe5, 0xab, 0x31,
    0x43, 0xdb, 0x5b, 0xfc, 0xe0, 0xfd, 0x10, 0x8e,
    0x4b, 0x82, 0xd1, 0x20, 0xa9, 0x3a, 0xd2, 0xca,
    0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
};

static const unsigned char
_BIDDH_RFC3526_3072_G[] = {
    0x02
};

/*
 * 5. 4096-bit MODP Group (Group 16)
 */
static const unsigned char
_BIDDH_RFC3526_4096_P[] = {
    0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
    0xc9, 0x0f, 0xda, 0xa2, 0x21, 0x68, 0xc2, 0x34,
    0xc4, 0xc6, 0x62, 0x8b, 0x80, 0xdc, 0x1c, 0xd1,
    0x29, 0x02, 0x4e, 0x08, 0x8a, 0x67, 0xcc, 0x74,
    0x02, 0x0b, 0xbe, 0xa6, 0x3b, 0x13, 0x9b, 0x22,
    0x51, 0x4a, 0x08, 0x79, 0x8e, 0x34, 0x04, 0xdd,
    0xef, 0x95, 0x19, 0xb3, 0xcd, 0x3a, 0x43, 0x1b,
    0x30, 0x2b, 0x0a, 0x6d, 0xf2, 0x5f, 0x14, 0x37,
    0x4f, 0xe1, 0x35, 0x6d, 0x6d, 0x51, 0xc2, 0x45,
    0xe4, 0x85, 0xb5, 0x76, 0x62, 0x5e, 0x7e, 0xc6,
    0xf4, 0x4c, 0x42, 0xe9, 0xa6, 0x37, 0xed, 0x6b,
    0x0b, 0xff, 0x5c, 0xb6, 0xf4, 0x06, 0xb7, 0xed,
    0xee, 0x38, 0x6b, 0xfb, 0x5a, 0x89, 0x9f, 0xa5,
    0xae, 0x9f, 0x24, 0x11, 0x7c, 0x4b, 0x1f, 0xe6,
    0x49, 0x28, 0x66, 0x51, 0xec, 0xe4, 0x5b, 0x3d,
    0xc2, 0x00, 0x7c, 0xb8, 0xa1, 0x63, 0xbf, 0x05,
    0x98, 0xda, 0x48, 0x36, 0x1c, 0x55, 0xd3, 0x9a,
    0x69, 0x16, 0x3f, 0xa8, 0xfd, 0x24, 0xcf, 0x5f,
    0x83, 0x65, 0x5d, 0x23, 0xdc, 0xa3, 0xad, 0x96,
    0x1c, 0x62, 0xf3, 0x56, 0x20, 0x85, 0x52, 0xbb,
    0x9e, 0xd5, 0x29, 0x07, 0x70, 0x96, 0x96, 0x6d,
    0x67, 0x0c, 0x35, 0x4e, 0x4a, 0xbc, 0x98, 0x04,
    0xf1, 0x74, 0x6c, 0x08, 0xca, 0x18, 0x21, 0x7c,
    0x32, 0x90, 0x5e, 0x46, 0x2e, 0x36, 0xce, 0x3b,
    0xe3, 0x9e, 0x77, 0x2c, 0x18, 0x0e, 0x86, 0x03,
    0x9b, 0x27, 0x83, 0xa2, 0xec, 0x07, 0xa2, 0x8f,
    0xb5, 0xc5, 0x5d, 0xf0, 0x6f, 0x4c, 0x52, 0xc9,
    0xde, 0x2b, 0xcb, 0xf6, 0x95, 0x58, 0x17, 0x18,
    0x39, 0x95, 0x49, 0x7c, 0xea, 0x95, 0x6a, 0xe5,
    0x15, 0xd2, 0x26, 0x18, 0x98, 0xfa, 0x05, 0x10,
    0x15, 0x72, 0x8e, 0x5a, 0x8a, 0xaa, 0xc4, 0x2d,
    0xad, 0x33, 0x17, 0x0d, 0x04, 0x50, 0x7a, 0x33,
    0xa8, 0x55, 0x21, 0xab, 0xdf, 0x1c, 0xba, 0x64,
    0xec, 0xfb, 0x85, 0x04, 0x58, 0xdb, 0xef, 0x0a,
    0x8a, 0xea, 0x71, 0x57, 0x5d, 0x06, 0x0c, 0x7d,
    0xb3, 0x97, 0x0f, 0x85, 0xa6, 0xe1, 0xe4, 0xc7,
    0xab, 0xf5, 0xae, 0x8c, 0xdb, 0x09, 0x33, 0xd7,
    0x1e, 0x8c, 0x94, 0xe0, 0x4a, 0x25, 0x61, 0x9d,
    0xce, 0xe3, 0xd2, 0x26, 0x1a, 0xd2, 0xee, 0x6b,
    0xf1, 0x2f, 0xfa, 0x06, 0xd9, 0x8a, 0x08, 0x64,
    0xd8, 0x76, 0x02, 0x73, 0x3e, 0xc8, 0x6a, 0x64,
    0x52, 0x1f, 0x2b, 0x18, 0x17, 0x7b, 0x20, 0x0c,
    0xbb, 0xe1, 0x17, 0x57, 0x7a, 0x61, 0x5d, 0x6c,
    0x77, 0x09, 0x88, 0xc0, 0xba, 0xd9, 0x46, 0xe2,
    0x08, 0xe2, 0x4f, 0xa0, 0x74, 0xe5, 0xab, 0x31,
    0x43, 0xdb, 0x5b, 0xfc, 0xe0, 0xfd, 0x10, 0x8e,
    0x4b, 0x82, 0xd1, 0x20, 0xa9, 0x21, 0x08, 0x01,
    0x1a, 0x72, 0x3c, 0x12, 0xa7, 0x87, 0xe6, 0xd7,
    0x88, 0x71, 0x9a, 0x10, 0xbd, 0xba, 0x5b, 0x26,
    0x99, 0xc3, 0x27, 0x18, 0x6a, 0xf4, 0xe2, 0x3c,
    0x1a, 0x94, 0x68, 0x34, 0xb6, 0x15, 0x0b, 0xda,
    0x25, 0x83, 0xe9, 0xca, 0x2a, 0xd4, 0x4c, 0xe8,
    0xdb, 0xbb, 0xc2, 0xdb, 0x04, 0xde, 0x8e, 0xf9,
    0x2e, 0x8e, 0xfc, 0x14, 0x1f, 0xbe, 0xca, 0xa6,
    0x28, 0x7c, 0x59, 0x47, 0x4e, 0x6b, 0xc0, 0x5d,
    0x99, 0xb2, 0x96, 0x4f, 0xa0, 0x90, 0xc3, 0xa2,
    0x23, 0x3b, 0xa1, 0x86, 0x51, 0x5b, 0xe7, 0xed,
    0x1f, 0x61, 0x29, 0x70, 0xce, 0xe2, 0xd7, 0xaf,
    0xb8, 0x1b, 0xdd, 0x76, 0x21, 0x70, 0x48, 0x1c,
    0xd0, 0x06, 0x91, 0x27, 0xd5, 0xb0, 0x5a, 0xa9,
    0x93, 0xb4, 0xea, 0x98, 0x8d, 0x8f, 0xdd, 0xc1,
    0x86, 0xff, 0xb7, 0xdc, 0x90, 0xa6, 0xc0, 0x8f,
    0x4d, 0xf4, 0x35, 0xc9, 0x34, 0x06, 0x31, 0x99,
    0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
};

static const unsigned char
_BIDDH_RFC3526_4096_G[] = {
    0x02
};

/*
 * 6. 6144-bit MODP Group (Group 17)
 */
static const unsigned char
_BIDDH_RFC3526_6144_P[] = {
    0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
    0xc9, 0x0f, 0xda, 0xa2, 0x21, 0x68, 0xc2, 0x34,
    0xc4, 0xc6, 0x62, 0x8b, 0x80, 0xdc, 0x1c, 0xd1,
    0x29, 0x02, 0x4e, 0x08, 0x8a, 0x67, 0xcc, 0x74,
    0x02, 0x0b, 0xbe, 0xa6, 0x3b, 0x13, 0x9b, 0x22,
    0x51, 0x4a, 0x08, 0x79, 0x8e, 0x34, 0x04, 0xdd,
    0xef, 0x95, 0x19, 0xb3, 0xcd, 0x3a, 0x43, 0x1b,
    0x30, 0x2b, 0x0a, 0x6d, 0xf2, 0x5f, 0x14, 0x37,
    0x4f, 0xe1, 0x35, 0x6d, 0x6d, 0x51, 0xc2, 0x45,
    0xe4, 0x85, 0xb5, 0x76, 0x62, 0x5e, 0x7e, 0xc6,
    0xf4, 0x4c, 0x42, 0xe9, 0xa6, 0x37, 0xed, 0x6b,
    0x0b, 0xff, 0x5c, 0xb6, 0xf4, 0x06, 0xb7, 0xed,
    0xee, 0x38, 0x6b, 0xfb, 0x5a, 0x89, 0x9f, 0xa5,
    0xae, 0x9f, 0x24, 0x11, 0x7c, 0x4b, 0x1f, 0xe6,
    0x49, 0x28, 0x66, 0x51, 0xec, 0xe4, 0x5b, 0x3d,
    0xc2, 0x00, 0x7c, 0xb8, 0xa1, 0x63, 0xbf, 0x05,
    0x98, 0xda, 0x48, 0x36, 0x1c, 0x55, 0xd3, 0x9a,
    0x69, 0x16, 0x3f, 0xa8, 0xfd, 0x24, 0xcf, 0x5f,
    0x83, 0x65, 0x5d, 0x23, 0xdc, 0xa3, 0xad, 0x96,
    0x1c, 0x62, 0xf3, 0x56, 0x20, 0x85, 0x52, 0xbb,
    0x9e, 0xd5, 0x29, 0x07, 0x70, 0x96, 0x96, 0x6d,
    0x67, 0x0c, 0x35, 0x4e, 0x4a, 0xbc, 0x98, 0x04,
    0xf1, 0x74, 0x6c, 0x08, 0xca, 0x18, 0x21, 0x7c,
    0x32, 0x90, 0x5e, 0x46, 0x2e, 0x36, 0xce, 0x3b,
    0xe3, 0x9e, 0x77, 0x2c, 0x18, 0x0e, 0x86, 0x03,
    0x9b, 0x27, 0x83, 0xa2, 0xec, 0x07, 0xa2, 0x8f,
    0xb5, 0xc5, 0x5d, 0xf0, 0x6f, 0x4c, 0x52, 0xc9,
    0xde, 0x2b, 0xcb, 0xf6, 0x95, 0x58, 0x17, 0x18,
    0x39, 0x95, 0x49, 0x7c, 0xea, 0x95, 0x6a, 0xe5,
    0x15, 0xd2, 0x26, 0x18, 0x98, 0xfa, 0x05, 0x10,
    0x15, 0x72, 0x8e, 0x5a, 0x8a, 0xaa, 0xc4, 0x2d,
    0xad, 0x33, 0x17, 0x0d, 0x04, 0x50, 0x7a, 0x33,
    0xa8, 0x55, 0x21, 0xab, 0xdf, 0x1c, 0xba, 0x64,
    0xec, 0xfb, 0x85, 0x04, 0x58, 0xdb, 0xef, 0x0a,
    0x8a, 0xea, 0x71, 0x57, 0x5d, 0x06, 0x0c, 0x7d,
    0xb3, 0x97, 0x0f, 0x85, 0xa6, 0xe1, 0xe4, 0xc7,
    0xab, 0xf5, 0xae, 0x8c, 0xdb, 0x09, 0x33, 0xd7,
    0x1e, 0x8c, 0x94, 0xe0, 0x4a, 0x25, 0x61, 0x9d,
    0xce, 0xe3, 0xd2, 0x26, 0x1a, 0xd2, 0xee, 0x6b,
    0xf1, 0x2f, 0xfa, 0x06, 0xd9, 0x8a, 0x08, 0x64,
    0xd8, 0x76, 0x02, 0x73, 0x3e, 0xc8, 0x6a, 0x64,
    0x52, 0x1f, 0x2b, 0x18, 0x17, 0x7b, 0x20, 0x0c,
    0xbb, 0xe1, 0x17, 0x57, 0x7a, 0x61, 0x5d, 0x6c,
    0x77, 0x09, 0x88, 0xc0, 0xba, 0xd9, 0x46, 0xe2,
    0x08, 0xe2, 0x4f, 0xa0, 0x74, 0xe5, 0xab, 0x31,
    0x43, 0xdb, 0x5b, 0xfc, 0xe0, 0xfd, 0x10, 0x8e,
    0x4b, 0x82, 0xd1, 0x20, 0xa9, 0x21, 0x08, 0x01,
    0x1a, 0x72, 0x3c, 0x12, 0xa7, 0x87, 0xe6, 0xd7,
    0x88, 0x71, 0x9a, 0x10, 0xbd, 0xba, 0x5b, 0x26,
    0x99, 0xc3, 0x27, 0x18, 0x6a, 0xf4, 0xe2, 0x3c,
    0x1a, 0x94, 0x68, 0x34, 0xb6, 0x15, 0x0b, 0xda,
    0x25, 0x83, 0xe9, 0xca, 0x2a, 0xd4, 0x4c, 0xe8,
    0xdb, 0xbb, 0xc2, 0xdb, 0x04, 0xde, 0x8e, 0xf9,
    0x2e, 0x8e, 0xfc, 0x14, 0x1f, 0xbe, 0xca, 0xa6,
    0x28, 0x7c, 0x59, 0x47, 0x4e, 0x6b, 0xc0, 0x5d,
    0x99, 0xb2, 0x96, 0x4f, 0xa0, 0x90, 0xc3, 0xa2,
    0x23, 0x3b, 0xa1, 0x86, 0x51, 0x5b, 0xe7, 0xed,
    0x1f, 0x61, 0x29, 0x70, 0xce, 0xe2, 0xd7, 0xaf,
    0xb8, 0x1b, 0xdd, 0x76, 0x21, 0x70, 0x48, 0x1c,
    0xd0, 0x06, 0x91, 0x27, 0xd5, 0xb0, 0x5a, 0xa9,
    0x93, 0xb4, 0xea, 0x98, 0x8d, 0x8f, 0xdd, 0xc1,
    0x86, 0xff, 0xb7, 0xdc, 0x90, 0xa6, 0xc0, 0x8f,
    0x4d, 0xf4, 0x35, 0xc9, 0x34, 0x02, 0x84, 0x92,
    0x36, 0xc3, 0xfa, 0xb4, 0xd2, 0x7c, 0x70, 0x26,
    0xc1, 0xd4, 0xdc, 0xb2, 0x60, 0x26, 0x46, 0xde,
    0xc9, 0x75, 0x1e, 0x76, 0x3d, 0xba, 0x37, 0xbd,
    0xf8, 0xff, 0x94, 0x06, 0xad, 0x9e, 0x53, 0x0e,
    0xe5, 0xdb, 0x38, 0x2f, 0x41, 0x30, 0x01, 0xae,
    0xb0, 0x6a, 0x53, 0xed, 0x90, 0x27, 0xd8, 0x31,
    0x17, 0x97, 0x27, 0xb0, 0x86, 0x5a, 0x89, 0x18,
    0xda, 0x3e, 0xdb, 0xeb, 0xcf, 0x9b, 0x14, 0xed,
    0x44, 0xce, 0x6c, 0xba, 0xce, 0xd4, 0xbb, 0x1b,
    0xdb, 0x7f, 0x14, 0x47, 0xe6, 0xcc, 0x25, 0x4b,
    0x33, 0x20, 0x51, 0x51, 0x2b, 0xd7, 0xaf, 0x42,
    0x6f, 0xb8, 0xf4, 0x01, 0x37, 0x8c, 0xd2, 0xbf,
    0x59, 0x83, 0xca, 0x01, 0xc6, 0x4b, 0x92, 0xec,
    0xf0, 0x32, 0xea, 0x15, 0xd1, 0x72, 0x1d, 0x03,
    0xf4, 0x82, 0xd7, 0xce, 0x6e, 0x74, 0xfe, 0xf6,
    0xd5, 0x5e, 0x70, 0x2f, 0x46, 0x98, 0x0c, 0x82,
    0xb5, 0xa8, 0x40, 0x31, 0x90, 0x0b, 0x1c, 0x9e,
    0x59, 0xe7, 0xc9, 0x7f, 0xbe, 0xc7, 0xe8, 0xf3,
    0x23, 0xa9, 0x7a, 0x7e, 0x36, 0xcc, 0x88, 0xbe,
    0x0f, 0x1d, 0x45, 0xb7, 0xff, 0x58, 0x5a, 0xc5,
    0x4b, 0xd4, 0x07, 0xb2, 0x2b, 0x41, 0x54, 0xaa,
    0xcc, 0x8f, 0x6d, 0x7e, 0xbf, 0x48, 0xe1, 0xd8,
    0x14, 0xcc, 0x5e, 0xd2, 0x0f, 0x80, 0x37, 0xe0,
    0xa7, 0x97, 0x15, 0xee, 0xf2, 0x9b, 0xe3, 0x28,
    0x06, 0xa1, 0xd5, 0x8b, 0xb7, 0xc5, 0xda, 0x76,
    0xf5, 0x50, 0xaa, 0x3d, 0x8a, 0x1f, 0xbf, 0xf0,
    0xeb, 0x19, 0xcc, 0xb1, 0xa3, 0x13, 0xd5, 0x5c,
    0xda, 0x56, 0xc9, 0xec, 0x2e, 0xf2, 0x96, 0x32,
    0x38, 0x7f, 0xe8, 0xd7, 0x6e, 0x3c, 0x04, 0x68,
    0x04, 0x3e, 0x8f, 0x66, 0x3f, 0x48, 0x60, 0xee,
    0x12, 0xbf, 0x2d, 0x5b, 0x0b, 0x74, 0x74, 0xd6,
    0xe6, 0x94, 0xf9, 0x1e, 0x6d, 0xcc, 0x40, 0x24,
    0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
};

static const unsigned char
_BIDDH_RFC3526_6144_G[] = {
    0x02
};

/*
 * 7. 8192-bit MODP Group (Group 18)
 */
static const unsigned char
_BIDDH_RFC3526_8192_P[] = {
    0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
    0xc9, 0x0f, 0xda, 0xa2, 0x21, 0x68, 0xc2, 0x34,
    0xc4, 0xc6, 0x62, 0x8b, 0x80, 0xdc, 0x1c, 0xd1,
    0x29, 0x02, 0x4e, 0x08, 0x8a, 0x67, 0xcc, 0x74,
    0x02, 0x0b, 0xbe, 0xa6, 0x3b, 0x13, 0x9b, 0x22,
    0x51, 0x4a, 0x08, 0x79, 0x8e, 0x34, 0x04, 0xdd,
    0xef, 0x95, 0x19, 0xb3, 0xcd, 0x3a, 0x43, 0x1b,
    0x30, 0x2b, 0x0a, 0x6d, 0xf2, 0x5f, 0x14, 0x37,
    0x4f, 0xe1, 0x35, 0x6d, 0x6d, 0x51, 0xc2, 0x45,
    0xe4, 0x85, 0xb5, 0x76, 0x62, 0x5e, 0x7e, 0xc6,
    0xf4, 0x4c, 0x42, 0xe9, 0xa6, 0x37, 0xed, 0x6b,
    0x0b, 0xff, 0x5c, 0xb6, 0xf4, 0x06, 0xb7, 0xed,
    0xee, 0x38, 0x6b, 0xfb, 0x5a, 0x89, 0x9f, 0xa5,
    0xae, 0x9f, 0x24, 0x11, 0x7c, 0x4b, 0x1f, 0xe6,
    0x49, 0x28, 0x66, 0x51, 0xec, 0xe4, 0x5b, 0x3d,
    0xc2, 0x00, 0x7c, 0xb8, 0xa1, 0x63, 0xbf, 0x05,
    0x98, 0xda, 0x48, 0x36, 0x1c, 0x55, 0xd3, 0x9a,
    0x69, 0x16, 0x3f, 0xa8, 0xfd, 0x24, 0xcf, 0x5f,
    0x83, 0x65, 0x5d, 0x23, 0xdc, 0xa3, 0xad, 0x96,
    0x1c, 0x62, 0xf3, 0x56, 0x20, 0x85, 0x52, 0xbb,
    0x9e, 0xd5, 0x29, 0x07, 0x70, 0x96, 0x96, 0x6d,
    0x67, 0x0c, 0x35, 0x4e, 0x4a, 0xbc, 0x98, 0x04,
    0xf1, 0x74, 0x6c, 0x08, 0xca, 0x18, 0x21, 0x7c,
    0x32, 0x90, 0x5e, 0x46, 0x2e, 0x36, 0xce, 0x3b,
    0xe3, 0x9e, 0x77, 0x2c, 0x18, 0x0e, 0x86, 0x03,
    0x9b, 0x27, 0x83, 0xa2, 0xec, 0x07, 0xa2, 0x8f,
    0xb5, 0xc5, 0x5d, 0xf0, 0x6f, 0x4c, 0x52, 0xc9,
    0xde, 0x2b, 0xcb, 0xf6, 0x95, 0x58, 0x17, 0x18,
    0x39, 0x95, 0x49, 0x7c, 0xea, 0x95, 0x6a, 0xe5,
    0x15, 0xd2, 0x26, 0x18, 0x98, 0xfa, 0x05, 0x10,
    0x15, 0x72, 0x8e, 0x5a, 0x8a, 0xaa, 0xc4, 0x2d,
    0xad, 0x33, 0x17, 0x0d, 0x04, 0x50, 0x7a, 0x33,
    0xa8, 0x55, 0x21, 0xab, 0xdf, 0x1c, 0xba, 0x64,
    0xec, 0xfb, 0x85, 0x04, 0x58, 0xdb, 0xef, 0x0a,
    0x8a, 0xea, 0x71, 0x57, 0x5d, 0x06, 0x0c, 0x7d,
    0xb3, 0x97, 0x0f, 0x85, 0xa6, 0xe1, 0xe4, 0xc7,
    0xab, 0xf5, 0xae, 0x8c, 0xdb, 0x09, 0x33, 0xd7,
    0x1e, 0x8c, 0x94, 0xe0, 0x4a, 0x25, 0x61, 0x9d,
    0xce, 0xe3, 0xd2, 0x26, 0x1a, 0xd2, 0xee, 0x6b,
    0xf1, 0x2f, 0xfa, 0x06, 0xd9, 0x8a, 0x08, 0x64,
    0xd8, 0x76, 0x02, 0x73, 0x3e, 0xc8, 0x6a, 0x64,
    0x52, 0x1f, 0x2b, 0x18, 0x17, 0x7b, 0x20, 0x0c,
    0xbb, 0xe1, 0x17, 0x57, 0x7a, 0x61, 0x5d, 0x6c,
    0x77, 0x09, 0x88, 0xc0, 0xba, 0xd9, 0x46, 0xe2,
    0x08, 0xe2, 0x4f, 0xa0, 0x74, 0xe5, 0xab, 0x31,
    0x43, 0xdb, 0x5b, 0xfc, 0xe0, 0xfd, 0x10, 0x8e,
    0x4b, 0x82, 0xd1, 0x20, 0xa9, 0x21, 0x08, 0x01,
    0x1a, 0x72, 0x3c, 0x12, 0xa7, 0x87, 0xe6, 0xd7,
    0x88, 0x71, 0x9a, 0x10, 0xbd, 0xba, 0x5b, 0x26,
    0x99, 0xc3, 0x27, 0x18, 0x6a, 0xf4, 0xe2, 0x3c,
    0x1a, 0x94, 0x68, 0x34, 0xb6, 0x15, 0x0b, 0xda,
    0x25, 0x83, 0xe9, 0xca, 0x2a, 0xd4, 0x4c, 0xe8,
    0xdb, 0xbb, 0xc2, 0xdb, 0x04, 0xde, 0x8e, 0xf9,
    0x2e, 0x8e, 0xfc, 0x14, 0x1f, 0xbe, 0xca, 0xa6,
    0x28, 0x7c, 0x59, 0x47, 0x4e, 0x6b, 0xc0, 0x5d,
    0x99, 0xb2, 0x96, 0x4f, 0xa0, 0x90, 0xc3, 0xa2,
    0x23, 0x3b, 0xa1, 0x86, 0x51, 0x5b, 0xe7, 0xed,
    0x1f, 0x61, 0x29, 0x70, 0xce, 0xe2, 0xd7, 0xaf,
    0xb8, 0x1b, 0xdd, 0x76, 0x21, 0x70, 0x48, 0x1c,
    0xd0, 0x06, 0x91, 0x27, 0xd5, 0xb0, 0x5a, 0xa9,
    0x93, 0xb4, 0xea, 0x98, 0x8d, 0x8f, 0xdd, 0xc1,
    0x86, 0xff, 0xb7, 0xdc, 0x90, 0xa6, 0xc0, 0x8f,
    0x4d, 0xf4, 0x35, 0xc9, 0x34, 0x02, 0x84, 0x92,
    0x36, 0xc3, 0xfa, 0xb4, 0xd2, 0x7c, 0x70, 0x26,
    0xc1, 0xd4, 0xdc, 0xb2, 0x60, 0x26, 0x46, 0xde,
    0xc9, 0x75, 0x1e, 0x76, 0x3d, 0xba, 0x37, 0xbd,
    0xf8, 0xff, 0x94, 0x06, 0xad, 0x9e, 0x53, 0x0e,
    0xe5, 0xdb, 0x38, 0x2f, 0x41, 0x30, 0x01, 0xae,
    0xb0, 0x6a, 0x53, 0xed, 0x90, 0x27, 0xd8, 0x31,
    0x17, 0x97, 0x27, 0xb0, 0x86, 0x5a, 0x89, 0x18,
    0xda, 0x3e, 0xdb, 0xeb, 0xcf, 0x9b, 0x14, 0xed,
    0x44, 0xce, 0x6c, 0xba, 0xce, 0xd4, 0xbb, 0x1b,
    0xdb, 0x7f, 0x14, 0x47, 0xe6, 0xcc, 0x25, 0x4b,
    0x33, 0x20, 0x51, 0x51, 0x2b, 0xd7, 0xaf, 0x42,
    0x6f, 0xb8, 0xf4, 0x01, 0x37, 0x8c, 0xd2, 0xbf,
    0x59, 0x83, 0xca, 0x01, 0xc6, 0x4b, 0x92, 0xec,
    0xf0, 0x32, 0xea, 0x15, 0xd1, 0x72, 0x1d, 0x03,
    0xf4, 0x82, 0xd7, 0xce, 0x6e, 0x74, 0xfe, 0xf6,
    0xd5, 0x5e, 0x70, 0x2f, 0x46, 0x98, 0x0c, 0x82,
    0xb5, 0xa8, 0x40, 0x31, 0x90, 0x0b, 0x1c, 0x9e,
    0x59, 0xe7, 0xc9, 0x7f, 0xbe, 0xc7, 0xe8, 0xf3,
    0x23, 0xa9, 0x7a, 0x7e, 0x36, 0xcc, 0x88, 0xbe,
    0x0f, 0x1d, 0x45, 0xb7, 0xff, 0x58, 0x5a, 0xc5,
    0x4b, 0xd4, 0x07, 0xb2, 0x2b, 0x41, 0x54, 0xaa,
    0xcc, 0x8f, 0x6d, 0x7e, 0xbf, 0x48, 0xe1, 0xd8,
    0x14, 0xcc, 0x5e, 0xd2, 0x0f, 0x80, 0x37, 0xe0,
    0xa7, 0x97, 0x15, 0xee, 0xf2, 0x9b, 0xe3, 0x28,
    0x06, 0xa1, 0xd5, 0x8b, 0xb7, 0xc5, 0xda, 0x76,
    0xf5, 0x50, 0xaa, 0x3d, 0x8a, 0x1f, 0xbf, 0xf0,
    0xeb, 0x19, 0xcc, 0xb1, 0xa3, 0x13, 0xd5, 0x5c,
    0xda, 0x56, 0xc9, 0xec, 0x2e, 0xf2, 0x96, 0x32,
    0x38, 0x7f, 0xe8, 0xd7, 0x6e, 0x3c, 0x04, 0x68,
    0x04, 0x3e, 0x8f, 0x66, 0x3f, 0x48, 0x60, 0xee,
    0x12, 0xbf, 0x2d, 0x5b, 0x0b, 0x74, 0x74, 0xd6,
    0xe6, 0x94, 0xf9, 0x1e, 0x6d, 0xbe, 0x11, 0x59,
    0x74, 0xa3, 0x92, 0x6f, 0x12, 0xfe, 0xe5, 0xe4,
    0x38, 0x77, 0x7c, 0xb6, 0xa9, 0x32, 0xdf, 0x8c,
    0xd8, 0xbe, 0xc4, 0xd0, 0x73, 0xb9, 0x31, 0xba,
    0x3b, 0xc8, 0x32, 0xb6, 0x8d, 0x9d, 0xd3, 0x00,
    0x74, 0x1f, 0xa7, 0xbf, 0x8a, 0xfc, 0x47, 0xed,
    0x25, 0x76, 0xf6, 0x93, 0x6b, 0xa4, 0x24, 0x66,
    0x3a, 0xab, 0x63, 0x9c, 0x5a, 0xe4, 0xf5, 0x68,
    0x34, 0x23, 0xb4, 0x74, 0x2b, 0xf1, 0xc9, 0x78,
    0x23, 0x8f, 0x16, 0xcb, 0xe3, 0x9d, 0x65, 0x2d,
    0xe3, 0xfd, 0xb8, 0xbe, 0xfc, 0x84, 0x8a, 0xd9,
    0x22, 0x22, 0x2e, 0x04, 0xa4, 0x03, 0x7c, 0x07,
    0x13, 0xeb, 0x57, 0xa8, 0x1a, 0x23, 0xf0, 0xc7,
    0x34, 0x73, 0xfc, 0x64, 0x6c, 0xea, 0x30, 0x6b,
    0x4b, 0xcb, 0xc8, 0x86, 0x2f, 0x83, 0x85, 0xdd,
    0xfa, 0x9d, 0x4b, 0x7f, 0xa2, 0xc0, 0x87, 0xe8,
    0x79, 0x68, 0x33, 0x03, 0xed, 0x5b, 0xdd, 0x3a,
    0x06, 0x2b, 0x3c, 0xf5, 0xb3, 0xa2, 0x78, 0xa6,
    0x6d, 0x2a, 0x13, 0xf8, 0x3f, 0x44, 0xf8, 0x2d,
    0xdf, 0x31, 0x0e, 0xe0, 0x74, 0xab, 0x6a, 0x36,
    0x45, 0x97, 0xe8, 0x99, 0xa0, 0x25, 0x5d, 0xc1,
    0x64, 0xf3, 0x1c, 0xc5, 0x08, 0x46, 0x85, 0x1d,
    0xf9, 0xab, 0x48, 0x19, 0x5d, 0xed, 0x7e, 0xa1,
    0xb1, 0xd5, 0x10, 0xbd, 0x7e, 0xe7, 0x4d, 0x73,
    0xfa, 0xf3, 0x6b, 0xc3, 0x1e, 0xcf, 0xa2, 0x68,
    0x35, 0x90, 0x46, 0xf4, 0xeb, 0x87, 0x9f, 0x92,
    0x40, 0x09, 0x43, 0x8b, 0x48, 0x1c, 0x6c, 0xd7,
    0x88, 0x9a, 0x00, 0x2e, 0xd5, 0xee, 0x38, 0x2b,
    0xc9, 0x19, 0x0d, 0xa6, 0xfc, 0x02, 0x6e, 0x47,
    0x95, 0x58, 0xe4, 0x47, 0x56, 0x77, 0xe9, 0xaa,
    0x9e, 0x30, 0x50, 0xe2, 0x76, 0x56, 0x94, 0xdf,
    0xc8, 0x1f, 0x56, 0xe8, 0x80, 0xb9, 0x6e, 0x71,
    0x60, 0xc9, 0x80, 0xdd, 0x98, 0xed, 0xd3, 0xdf,
    0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
};

static const unsigned char
_BIDDH_RFC3526_8192_G[] = {
    0x02
};


static struct BIDDHGroupDesc {
    size_t cbPrime;
    const unsigned char *pbPrime;
    size_t cbGenerator;
    const unsigned char *pbGenerator;
} _BIDDHGroups[] = {
    {
        sizeof(_BIDDH_RFC5114_1024_P),
        _BIDDH_RFC5114_1024_P,
        sizeof(_BIDDH_RFC5114_1024_G),
        _BIDDH_RFC5114_1024_G,
    },
    {
        sizeof(_BIDDH_RFC5114_2048_P),
        _BIDDH_RFC5114_2048_P,
        sizeof(_BIDDH_RFC5114_2048_G),
        _BIDDH_RFC5114_2048_G,
    },
    {
        sizeof(_BIDDH_RFC3526_3072_P),
        _BIDDH_RFC3526_3072_P,
        sizeof(_BIDDH_RFC3526_3072_G),
        _BIDDH_RFC3526_3072_G,
    },
    {
        sizeof(_BIDDH_RFC3526_4096_P),
        _BIDDH_RFC3526_4096_P,
        sizeof(_BIDDH_RFC3526_4096_G),
        _BIDDH_RFC3526_4096_G,
    },
    {
        sizeof(_BIDDH_RFC3526_6144_P),
        _BIDDH_RFC3526_6144_P,
        sizeof(_BIDDH_RFC3526_6144_G),
        _BIDDH_RFC3526_6144_G,
    },
    {
        sizeof(_BIDDH_RFC3526_8192_P),
        _BIDDH_RFC3526_8192_P,
        sizeof(_BIDDH_RFC3526_8192_G),
        _BIDDH_RFC3526_8192_G,
    },
};

BIDError
_BIDGetFixedDHParams(
    BIDContext context,
    json_t **pDhParams)
{
    BIDError err;
    json_t *dhParams = NULL;
    static struct BIDDHGroupDesc *dhGroup = NULL;
    size_t i;
    int bFound = 0;

    for (i = 0; i < sizeof(_BIDDHGroups) / sizeof(_BIDDHGroups[0]); i++) {
        dhGroup = &_BIDDHGroups[i];

        if (dhGroup->cbPrime * 8 == context->DHKeySize) {
            bFound++;
            break;
        }
    }

    if (!bFound) {
        err = BID_S_DH_PARAM_GENERATION_FAILURE;
        goto cleanup;
    }

    dhParams = json_object();
    if (dhParams == NULL) {
        err = BID_S_NO_MEMORY;
        goto cleanup;
    }

    err = _BIDJsonObjectSetBinaryValue(context, dhParams, "g",
                                       dhGroup->pbGenerator,
                                       dhGroup->cbGenerator);
    BID_BAIL_ON_ERROR(err);

    err = _BIDJsonObjectSetBinaryValue(context, dhParams, "p",
                                       dhGroup->pbPrime,
                                       dhGroup->cbPrime);
    BID_BAIL_ON_ERROR(err);

    *pDhParams = dhParams;

cleanup:
    if (err != BID_S_OK)
        json_decref(dhParams);

    return err;
}
